@@ -654,19 +654,6 @@ public class AggregationTreeTest {
         return fCsa;
     }
 
-    /**
-     * Set the call graph analysis
-     *
-     * @param cga
-     *            The call graph analysis
-     * @deprecated Use {@link #setCga(CGAnalysis, ITmfStateSystem)} instead
-     */
-    @Deprecated
-    protected void setCga(CGAnalysis cga) {
-        fCsa = new CallStackAnalysisStub();
-        fCga = cga;
-    }
-
     /**
      * Set the analyses to null
      */
